/*
 * Overview access routines and data
 *
 * @(#)$Header: overview.h,v 1.0 93/09/27 15:55:21 davison Exp $
 */

#define OVER_NAME ".overview"

/* Undefine this if a new field makes the first letters no longer unique. */
#define OVER_UNIQUE_1ST_LTRS

char *over_field[] = {
	"ARTICLE #",		/* 0 */
	"subject",		/* 1 */
	"from",			/* 2 */
	"date",			/* 3 */
	"message-id",		/* 4 */
	"references",		/* 5 */
	"bytes",		/* 6 */
	"lines"			/* 7 */
#ifdef OVER_XREFS
	,"xref"			/* 8 */
#endif
};

/* Customize this count if you add fields to the above list. */
#ifdef OVER_XREFS
#define OVER_LAST_FIELD 8
#else
#define OVER_LAST_FIELD 7
#endif

void over_fake();
void over_close();
